
/* Chrysalide - Outil d'analyse de fichiers binaires
 * context-int.h - prototypes internes pour un suivi d'analyses via contextes
 *
 * Copyright (C) 2022 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef _ANALYSIS_SCAN_CONTEXT_INT_H
#define _ANALYSIS_SCAN_CONTEXT_INT_H


#include "context.h"


#include "expr.h"
#include "../../common/fnv1a.h"
#include "../../glibext/umemslice.h"


//#define __USE_TABLE_FOR_MATCHES


#ifndef __USE_TABLE_FOR_MATCHES

/* Lien entre un motif et ses correspondances */
typedef struct _matched_pattern_t
{
    const GSearchPattern *pattern;          /* Motif recherché             */
    GScanMatches *matches;                  /* Correspondances associées   */

} matched_pattern_t;

#endif

/* Condition définissant une règle de correspondance */
typedef struct _rule_condition_t
{
    char *name;                             /* Désignation de la règle     */
    fnv64_t name_hash;                      /* Empreinte de la désignation */

    GScanExpression *expr;                  /* Condition de correspondance */
    bool final_reduced;                     /* Réduction finale tentée ?   */

} rule_condition_t;

/* Contexte de suivi d'une analyse en cours (instance) */
struct _GScanContext
{
    GObject parent;                         /* A laisser en premier        */

    GScanOptions *options;                  /* Options d'analyses          */

    GBinContent *content;                   /* Contenu binaire traité      */
    bool scan_done;                         /* Phase d'analyse terminée ?  */

    GUMemSlice *match_allocator;            /* Suivi de correspondances    */
    match_area_t **match_storages;          /* Suivi de correspondances    */
    size_t storages_count;                  /* Quantité de ces suivis      */

#ifdef __USE_TABLE_FOR_MATCHES
    GHashTable *full_trackers;              /* Correspondances confirmées  */
#else
    matched_pattern_t *full_trackers;       /* Correspondances confirmées  */
    size_t full_allocated;                  /* Quantité d'éléments alloués */
    size_t full_count;                      /* Quantité de correspondances */
#endif

    bool global;                            /* Validation globale          */

    rule_condition_t *conditions;           /* Ensemble de règles suivies  */
    size_t cond_count;                      /* Quantité de ces conditions  */

};

/* Contexte de suivi d'une analyse en cours (classe) */
struct _GScanContextClass
{
    GObjectClass parent;                    /* A laisser en premier        */

};



#endif  /* _ANALYSIS_SCAN_CONTEXT_INT_H */
